use super::super::types::set_member_value;
use tool_helper::{bits::BitRange, raw::RawConversion};

#[repr(packed(1))]
#[allow(dead_code)]
pub struct Header {
    value: u32
}

#[allow(dead_code)]
impl Header {
    const TEX_WIDTH_BIT_RANGE:   BitRange = BitRange::from_to(0, 8);
    const TEX_HEIGHT_BIT_RANGE:  BitRange = BitRange::from_to(9, 16);
    const CLUT_WIDTH_BIT_RANGE:  BitRange = BitRange::from_to(17, 22);
    const CLUT_HEIGHT_BIT_RANGE: BitRange = BitRange::from_to(23, 31);

    pub fn encode(tex_width: u16, tex_height: u16, clut_width: u16, clut_height: u16) -> Option<Header> {
        if tex_width & 1 == 1 || tex_height & 1 == 1 {
            None
        }

        else {
            let value = set_member_value!(set_member_value!(set_member_value!(set_member_value!(0, 
                tex_width,   1, u32),
                tex_height,  1, u32),
                clut_width,  4, u32),
                clut_height, 0, u32);

            Some(Header{value})
        }
    }
}

impl RawConversion<4> for Header {
    fn convert_to_raw(&self) -> [u8; 4] {
        self.value.to_le_bytes()
    }
}